<?php
/**
 * bPost AtIntlHome class
 *
 * @author    Serge Jamasb <serge@stigmi.eu>
 * @version   3.3.9
 * @copyright Copyright (c), Eontech.net. All rights reserved.
 * @license   BSD License
 */

class EontechModBpostOrderBoxAtIntlHome extends EontechModBpostOrderBoxInternational
{
	/**
	 * @param string $product
	 * @throws EontechModException
	 */
	public function setProduct($product)
	{
		if (!in_array($product, self::getPossibleProductValues()))
			throw new EontechModException(
				sprintf(
					'Invalid value, possible values are: %1$s.',
					implode(', ', self::getPossibleProductValues())
				)
			);

		parent::setProduct($product);
	}

	/**
	 * @return array
	 */
	public static function getPossibleProductValues()
	{
		return array(
			'bpack World Business',
			'bpack World Express Pro',
			'bpack Europe Business',
			'bpack World Easy Return',
		);
	}

	/**
	 * Return the object as an array for usage in the XML
	 *
	 * @param  \DomDocument $document
	 * @param  string	   $prefix
	 * @return \DomElement
	 */
	public function toXML(\DOMDocument $document, $prefix = null, $type = null)
	{
		$tag_name = 'internationalBox';
		if ($prefix !== null)
			$tag_name = $prefix.':'.$tag_name;

		$international_box = $document->createElement($tag_name);
		$international = parent::toXML($document, 'international', 'atIntlHome');
		$international_box->appendChild($international);

		return $international_box;
	}

	/**
	 * @param  \SimpleXMLElement $xml
	 * @return EontechModBpostOrderBoxInternational
	 * @throws EontechModException
	 */
	public static function createFromXML(\SimpleXMLElement $xml)
	{
		$at_intl_home = new EontechModBpostOrderBoxAtIntlHome();

		if (isset($xml->atIntlHome->product) && $xml->atIntlHome->product != '')
			$at_intl_home->setProduct((string)$xml->atIntlHome->product);
		if (isset($xml->atIntlHome->options))
			foreach ($xml->atIntlHome->options as $option_data)
			{
				$option_data = $option_data->children('http://schema.post.be/shm/deepintegration/v3/common');

				if (in_array($option_data->getName(), array('infoDistributed')))
					$option = EontechModBpostOrderBoxOptionMessaging::createFromXML($option_data);
				else
				{
					$class_name = 'EontechModBpostOrderBoxOption'.\Tools::ucfirst($option_data->getName());
					if (!method_exists($class_name, 'createFromXML'))
						throw new EontechModException('Not Implemented');
					$option = call_user_func(
						array($class_name, 'createFromXML'),
						$option_data
					);
				}

				$at_intl_home->addOption($option);
			}
		if (isset($xml->atIntlHome->parcelWeight) && $xml->atIntlHome->parcelWeight != '')
			$at_intl_home->setParcelWeight((int)$xml->atIntlHome->parcelWeight);
		if (isset($xml->atIntlHome->receiver))
		{
			$receiver_data = $xml->atIntlHome->receiver->children(
				'http://schema.post.be/shm/deepintegration/v3/common'
			);
			$at_intl_home->setReceiver(
				EontechModBpostOrderReceiver::createFromXML($receiver_data)
			);
		}
		if (isset($xml->atIntlHome->customsInfo))
			$at_intl_home->setCustomsInfo(EontechModBpostOrderBoxCustomsinfoCustomsInfo::createFromXML($xml->atIntlHome->customsInfo));

		return $at_intl_home;
	}
}
