<?php
/**
 * bPost AtIntlPugo class
 *
 * @author    Serge Jamasb <serge@stigmi.eu>
 * @version   3.3.9
 * @copyright Copyright (c), Eontech.net. All rights reserved.
 * @license   BSD License
 */

class EontechModBpostOrderBoxAtIntlPugo extends EontechModBpostOrderBoxInternational
{
	/**
	 * @var string
	 */
	protected $product = 'bpack@bpost international';

	/**
	 * @var string
	 */
	private $pugo_id;

	/**
	 * @var string
	 */
	private $pugo_name;

	/**
	 * @var EontechModBpostOrderIntlPugoAddress;
	 */
	private $pugo_address;

	/**
	 * @param string $product Possible values are: bpack@bpost international
	 * @throws EontechModException
	 */
	public function setProduct($product)
	{
		if (!in_array($product, self::getPossibleProductValues()))
			throw new EontechModException(
				sprintf(
					'Invalid value, possible values are: %1$s.',
					implode(', ', self::getPossibleProductValues())
				)
			);

		parent::setProduct($product);
	}

	/**
	 * @return array
	 */
	public static function getPossibleProductValues()
	{
		return array(
			'bpack@bpost international',
		);
	}

	/**
	 * @param string $pugo_id
	 */
	public function setPugoId($pugo_id)
	{
		$this->pugo_id = $pugo_id;
	}

	/**
	 * @return string
	 */
	public function getPugoId()
	{
		return $this->pugo_id;
	}

	/**
	 * @param string $pugo_name
	 */
	public function setPugoName($pugo_name)
	{
		$this->pugo_name = $pugo_name;
	}

	/**
	 * @return string
	 */
	public function getPugoName()
	{
		return $this->pugo_name;
	}

	/**
	 * @param EontechModBpostOrderIntlPugoAddress $pugo_address
	 */
	public function setPugoAddress($pugo_address)
	{
		$this->pugo_address = $pugo_address;
	}

	/**
	 * @return EontechModBpostOrderIntlPugoAddress
	 */
	public function getPugoAddress()
	{
		return $this->pugo_address;
	}

	/**
	 * Return the object as an array for usage in the XML
	 *
	 * @param  \DomDocument $document
	 * @param  string	   $prefix
	 * @param  string	   $type
	 * @return \DomElement
	 */
	public function toXML(\DOMDocument $document, $prefix = null, $type = null)
	{
		$tag_name = 'internationalBox';
		if ($prefix !== null)
			$tag_name = $prefix.':'.$tag_name;

		$international_box = $document->createElement($tag_name);
		$box_element = parent::toXML($document, 'international', 'atIntlPugo');
		$international_box->appendChild($box_element);

		if ($this->getPugoId() !== null)
		{
			$tag_name = 'international:pugoId';
			$box_element->appendChild(
				$document->createElement(
					$tag_name,
					$this->getPugoId()
				)
			);
		}
		if ($this->getPugoName() !== null)
		{
			$tag_name = 'international:pugoName';
			$box_element->appendChild(
				$document->createElement(
					$tag_name,
					$this->getPugoName()
				)
			);
		}
		if ($this->getPugoAddress() !== null)
			$box_element->appendChild(
				$this->getPugoAddress()->toXML($document, 'common')
			);

		return $international_box;
	}

	/**
	 * @param  \SimpleXMLElement $xml
	 * @return EontechModBpostOrderBoxAtIntlPugo
	 * @throws EontechModException
	 */
	public static function createFromXML(\SimpleXMLElement $xml)
	{
		$at_intl_pugo = new EontechModBpostOrderBoxAtIntlPugo();

		if (isset($xml->atIntlPugo->product) && $xml->atIntlPugo->product != '')
			$at_intl_pugo->setProduct((string)$xml->atIntlPugo->product);
		if (isset($xml->atIntlPugo->options))
			foreach ($xml->atIntlPugo->options as $option_data)
			{
				$option_data = $option_data->children('http://schema.post.be/shm/deepintegration/v3/common');

				if (in_array(
					$option_data->getName(),
					array(
						'infoDistributed',
						'infoNextDay',
						'infoReminder',
						'keepMeInformed',
					)))
					$option = EontechModBpostOrderBoxOptionMessaging::createFromXML($option_data);
				else
				{
					$class_name = 'EontechModBpostOrderBoxOption'.\Tools::ucfirst($option_data->getName());
					if (!method_exists($class_name, 'createFromXML'))
						throw new EontechModException('Not Implemented');
					$option = call_user_func(
						array($class_name, 'createFromXML'),
						$option_data
					);
				}

				$at_intl_pugo->addOption($option);
			}
		if (isset($xml->atIntlPugo->parcelWeight) && $xml->atIntlPugo->parcelWeight != '')
			$at_intl_pugo->setParcelWeight((int)$xml->atIntlPugo->parcelWeight);
		if (isset($xml->atIntlPugo->receiver))
		{
			$receiver_data = $xml->atIntlPugo->receiver->children(
				'http://schema.post.be/shm/deepintegration/v3/common'
			);
			$at_intl_pugo->setReceiver(
				EontechModBpostOrderReceiver::createFromXML($receiver_data)
			);
		}
		if (isset($xml->atIntlPugo->customsInfo))
			$at_intl_pugo->setCustomsInfo(EontechModBpostOrderBoxCustomsinfoCustomsInfo::createFromXML($xml->atIntlPugo->customsInfo));


		if (isset($xml->atIntlPugo->pugoId) && $xml->atIntlPugo->pugoId != '')
			$at_intl_pugo->setPugoId((string)$xml->atIntlPugo->pugoId);
		if (isset($xml->atIntlPugo->pugoName) && $xml->atIntlPugo->pugoName != '')
			$at_intl_pugo->setPugoName((string)$xml->atIntlPugo->pugoName);
		if (isset($xml->atIntlPugo->pugoAddress))
		{
			$pugo_address_data = $xml->atIntlPugo->pugoAddress->children(
				'http://schema.post.be/shm/deepintegration/v3/common'
			);
			$at_intl_pugo->setPugoAddress(
				EontechModBpostOrderIntlPugoAddress::createFromXML($pugo_address_data)
			);
		}

		return $at_intl_pugo;
	}
}