<?php
/**
 * bPost International base class
 *
 * @author    Serge Jamasb <serge@stigmi.eu>
 * @version   3.3.9
 * @copyright Copyright (c), Eontech.net. All rights reserved.
 * @license   BSD License
 */

abstract class EontechModBpostOrderBoxInternational
{
	/**
	 * @var string
	 */
	protected $product;

	/**
	 * @var array
	 */
	protected $options;

	/**
	 * @var EontechModBpostOrderReceiver
	 */
	protected $receiver;

	/**
	 * @var int
	 */
	protected $parcel_weight;

	/**
	 * @var EontechModBpostOrderBoxCustomsinfoCustomsInfo
	 */
	protected $customs_info;

	/**
	 * @param string $product
	 */
	public function setProduct($product)
	{
		$this->product = $product;
	}

	/**
	 * @return string
	 */
	public function getProduct()
	{
		return $this->product;
	}

	/**
	 * @remark should be implemented by the child class
	 * @return array
	 */
	public static function getPossibleProductValues()
	{
		return array();
	}

	/**
	 * @param EontechModBpostOrderBoxCustomsinfoCustomsInfo $customs_info
	 */
	public function setCustomsInfo($customs_info)
	{
		$this->customs_info = $customs_info;
	}

	/**
	 * @return EontechModBpostOrderBoxCustomsinfoCustomsInfo
	 */
	public function getCustomsInfo()
	{
		return $this->customs_info;
	}

	/**
	 * @param array $options
	 */
	public function setOptions($options)
	{
		$this->options = $options;
	}

	/**
	 * @return array
	 */
	public function getOptions()
	{
		return $this->options;
	}

	/**
	 * @param EontechModBpostOrderBoxOption $option
	 */
	public function addOption(EontechModBpostOrderBoxOption $option)
	{
		$this->options[] = $option;
	}

	/**
	 * @param int $parcel_weight
	 */
	public function setParcelWeight($parcel_weight)
	{
		$this->parcel_weight = $parcel_weight;
	}

	/**
	 * @return int
	 */
	public function getParcelWeight()
	{
		return $this->parcel_weight;
	}

	/**
	 * @param EontechModBpostOrderReceiver $receiver
	 */
	public function setReceiver($receiver)
	{
		$this->receiver = $receiver;
	}

	/**
	 * @return EontechModBpostOrderReceiver
	 */
	public function getReceiver()
	{
		return $this->receiver;
	}

	/**
	 * Return the object as an array for usage in the XML
	 *
	 * @param  \DomDocument $document
	 * @param  string	   $prefix
	 * @param  string	   $type
	 * @return \DomElement
	 */
	public function toXML(\DOMDocument $document, $prefix = null, $type = null)
	{
		$tag_prefix = ($prefix !== null) ? $prefix.':' : '';
		$type_element = $document->createElement($tag_prefix.$type);

		if ($this->getProduct() !== null)
		{
			$tag_name = $tag_prefix.'product';
			$type_element->appendChild(
				$document->createElement(
					$tag_name,
					$this->getProduct()
				)
			);
		}

		$options = $this->getOptions();
		if (!empty($options))
		{
			$options_element = $document->createElement($tag_prefix.'options');
			foreach ($options as $option)
				$options_element->appendChild($option->toXML($document));
			$type_element->appendChild($options_element);
		}

		if ($this->getReceiver() !== null)
			$type_element->appendChild($this->getReceiver()->toXML($document, $prefix));
			// $type_element->appendChild($this->getReceiver()->toXML($document, 'international'));

		if ($this->getParcelWeight() !== null)
			$type_element->appendChild(
				$document->createElement(
					$tag_prefix.'parcelWeight',
					$this->getParcelWeight()
				)
			);

		if ($this->getCustomsInfo() !== null)
			$type_element->appendChild($this->getCustomsInfo()->toXML($document, $prefix));

		return $type_element;
	}

	/**
	 * @param  \SimpleXMLElement $xml
	 * @return
	 */
	public static function createFromXML(\SimpleXMLElement $xml)
	{
	}
}
